import { warning, isIterator } from './util';
import { makeStateLogger, makeActionLogger } from './logger';

export function wrapGenerator(genFn) {
  for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  return function (scope) {
    var gen = genFn.apply(undefined, [scope].concat(args));

    var _gen$next = gen.next(),
        done = _gen$next.done;

    warning(!done, 'wrapGenerator: generator function already completed!');

    return gen;
  };
}

export var generatorMiddleware = function generatorMiddleware(next) {
  return function (initialState, resetFn, actionFn) {
    var wrappedActionFn = function wrappedActionFn(scope, actionHandler, params, actionPath) {
      if (isIterator(actionHandler)) actionFn(scope, function (scope) {
        for (var _len2 = arguments.length, params = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
          params[_key2 - 1] = arguments[_key2];
        }

        return actionHandler.next(params);
      }, params, actionPath);else actionFn(scope, actionHandler, params, actionPath);
    };

    next(initialState, resetFn, wrappedActionFn);
  };
};

export var applyLogger = function applyLogger(stateLogger, actionLogger) {
  if (typeof stateLogger === 'undefined') stateLogger = makeStateLogger();
  if (typeof actionLogger === 'undefined') actionLogger = makeActionLogger();

  return function (next) {
    return function (initialState, resetFn, actionFn) {
      var wrappedResetFn = function wrappedResetFn(oldState, newState) {
        if (stateLogger) stateLogger(oldState, newState);

        resetFn(oldState, newState);
      };

      var wrappedActionFn = function wrappedActionFn(scope, actionHandler, params, actionPath) {
        if (actionLogger) actionLogger(scope.atom, actionPath, params);

        actionFn(scope, actionHandler, params, actionPath);
      };

      next(initialState, wrappedResetFn, wrappedActionFn);
    };
  };
};