var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

export function warning(condition, message) {
  if (!condition && (typeof console === 'undefined' ? 'undefined' : _typeof(console)) !== undefined) {
    console.error('Warning: ' + message);
    try {
      // This error was thrown as a convenience so that you can use this stack
      // to find the callsite that caused this warning to fire.
      throw new Error(message);
    } catch (e) {}
  }
}

export function mapObjIndexed(fn, obj) {
  return Object.keys(obj || {}).reduce(function (r, key) {
    var _Object$assign;

    return Object.assign(r, (_Object$assign = {}, _Object$assign[key] = fn(obj[key], key), _Object$assign));
  }, {});
}

export function mapObj(fn, obj) {
  return Object.keys(obj || {}).reduce(function (r, key) {
    var _Object$assign2;

    return Object.assign(r, (_Object$assign2 = {}, _Object$assign2[key] = fn(obj[key]), _Object$assign2));
  }, {});
}

export function compose() {
  for (var _len = arguments.length, fns = Array(_len), _key = 0; _key < _len; _key++) {
    fns[_key] = arguments[_key];
  }

  var outerFn = fns.pop();

  return function () {
    return fns.reduceRight(function (acc, fn) {
      return fn(acc);
    }, outerFn.apply(undefined, arguments));
  };
}

// XXX: wrap in currying goodness
// XXX: check transpilation of { ...obj }
export var assoc = function assoc(p) {
  return function (val, obj) {
    var _extends2;

    return _extends({}, obj, (_extends2 = {}, _extends2[p] = val, _extends2));
  };
};
export var prop = function prop(p) {
  return function (obj) {
    return obj[p];
  };
};

export var isIterator = function isIterator(o) {
  return o && typeof o.next === 'function';
};

// Simple van Laarhoven lens implementation

var fmap = function fmap(fn, functor) {
  return functor.map(fn);
};

var Identity = function Identity(val) {
  return {
    val: val,
    map: function map(f) {
      return Identity(f(val));
    }
  };
};

var Constantly = function Constantly(val) {
  return {
    val: val,
    map: function map(f) {
      return this;
    }
  };
};

export var lens = function lens(getter, setter) {
  return function (vf) {
    return function (in_) {
      return fmap(function (focus) {
        return setter(focus, in_);
      }, vf(getter(in_)));
    };
  };
};

export var view = function view(lens, x) {
  return lens(Constantly)(x).val;
};
export var over = function over(lens, f, x) {
  return lens(function (v) {
    return Identity(f(v));
  })(x).val;
};
export var set = function set(lens, v, x) {
  return over(lens, function () {
    return v;
  }, x);
};

export var lensProp = function lensProp(p) {
  return lens(prop(p), assoc(p));
};