import { warning, isIterator } from './util';
import { makeStateLogger, makeActionLogger } from './logger';

export function wrapGenerator(genFn, ...args) {
  return function (scope) {
    const gen = genFn(scope, ...args);
    const { done } = gen.next();

    warning(!done, 'wrapGenerator: generator function already completed!');

    return gen;
  };
}

export const generatorMiddleware = next => (initialState, resetFn, actionFn) => {
  const wrappedActionFn = (scope, actionHandler, params, actionPath) => {
    if (isIterator(actionHandler)) actionFn(scope, (scope, ...params) => actionHandler.next(params), params, actionPath);else actionFn(scope, actionHandler, params, actionPath);
  };

  next(initialState, resetFn, wrappedActionFn);
};

export const applyLogger = (stateLogger, actionLogger) => {
  if (typeof stateLogger === 'undefined') stateLogger = makeStateLogger();
  if (typeof actionLogger === 'undefined') actionLogger = makeActionLogger();

  return next => (initialState, resetFn, actionFn) => {
    const wrappedResetFn = (oldState, newState) => {
      if (stateLogger) stateLogger(oldState, newState);

      resetFn(oldState, newState);
    };

    const wrappedActionFn = (scope, actionHandler, params, actionPath) => {
      if (actionLogger) actionLogger(scope.atom, actionPath, params);

      actionFn(scope, actionHandler, params, actionPath);
    };

    next(initialState, wrappedResetFn, wrappedActionFn);
  };
};