import React from 'react';
import PropTypes from 'prop-types';
import { Atom } from './atom';

export class Provider extends React.Component {

  getChildContext() {
    return { stateAtom: this.stateAtom };
  }

  constructor(props, context) {
    super(props, context);
    this.stateAtom = props.stateAtom;
  }

  render() {
    return React.Children.only(this.props.children);
  }
}

Provider.propTypes = {
  stateAtom: PropTypes.instanceOf(Atom),
  children: PropTypes.node.isRequired
};
Provider.childContextTypes = {
  stateAtom: PropTypes.instanceOf(Atom)
};
const getDisplayName = WrappedComponent => WrappedComponent.displayName || WrappedComponent.name || 'Component';

export function connect(mapStateFn, mapActionsFn, equalFn = null) {
  if (typeof equalFn !== 'function') equalFn = (a, b) => a === b;

  return function connectHOC(WrappedComponent) {
    var _class, _temp;

    return _temp = _class = class extends React.PureComponent {

      constructor(props, context) {
        super(props, context);

        this.stateUpdateHandler = () => {
          if (mapStateFn) {
            const newState = mapStateFn(this.context.stateAtom.deref(), this.props);
            if (!equalFn(this.state, newState)) this.setState(newState);
          }
        };

        if (mapStateFn) this.state = mapStateFn(context.stateAtom.deref(), props);
        if (mapActionsFn) this.actions = mapActionsFn(context.stateAtom, props);

        context.stateAtom.addWatch(this.stateUpdateHandler);
      }

      componentDidMount() {
        this.refreshActions(this.props);
        this.stateUpdateHandler();
      }

      componentWillReceiveProps(nextProps) {
        this.refreshActions(nextProps);
        if (mapStateFn) this.setState(mapStateFn(this.context.stateAtom.deref(), nextProps));
      }

      refreshActions(props) {
        if (mapActionsFn) this.actions = mapActionsFn(this.context.stateAtom, props);
      }

      componentWillUnmount() {
        this.context.stateAtom.removeWatch(this.stateUpdateHandler);
      }

      render() {
        return React.createElement(WrappedComponent, Object.assign({}, this.props, this.actions, this.state));
      }
    }, _class.displayName = `BindAtom(${getDisplayName(WrappedComponent)})`, _class.contextTypes = {
      stateAtom: PropTypes.instanceOf(Atom)
    }, _temp;
  };
}

// const mapState = (state, ownProps) => ({
//  notes: state.notes,
// })
//
// const mapActions = (atom, ownProps) => ({
//   addNote: (...args) => atom.dispatch('notes.add', ...args),
// })
//
// export default connect(mapState, mapActions)(SomeComponent)