'use strict';

exports.__esModule = true;
exports.applyLogger = exports.generatorMiddleware = undefined;
exports.wrapGenerator = wrapGenerator;

var _util = require('./util');

var _logger = require('./logger');

function wrapGenerator(genFn) {
  for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  return function (scope) {
    var gen = genFn.apply(undefined, [scope].concat(args));

    var _gen$next = gen.next(),
        done = _gen$next.done;

    (0, _util.warning)(!done, 'wrapGenerator: generator function already completed!');

    return gen;
  };
}

var generatorMiddleware = exports.generatorMiddleware = function generatorMiddleware(next) {
  return function (initialState, resetFn, actionFn) {
    var wrappedActionFn = function wrappedActionFn(scope, actionHandler, params, actionPath) {
      if ((0, _util.isIterator)(actionHandler)) actionFn(scope, function (scope) {
        for (var _len2 = arguments.length, params = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
          params[_key2 - 1] = arguments[_key2];
        }

        return actionHandler.next(params);
      }, params, actionPath);else actionFn(scope, actionHandler, params, actionPath);
    };

    next(initialState, resetFn, wrappedActionFn);
  };
};

var applyLogger = exports.applyLogger = function applyLogger(stateLogger, actionLogger) {
  if (typeof stateLogger === 'undefined') stateLogger = (0, _logger.makeStateLogger)();
  if (typeof actionLogger === 'undefined') actionLogger = (0, _logger.makeActionLogger)();

  return function (next) {
    return function (initialState, resetFn, actionFn) {
      var wrappedResetFn = function wrappedResetFn(oldState, newState) {
        if (stateLogger) stateLogger(oldState, newState);

        resetFn(oldState, newState);
      };

      var wrappedActionFn = function wrappedActionFn(scope, actionHandler, params, actionPath) {
        if (actionLogger) actionLogger(scope.atom, actionPath, params);

        actionFn(scope, actionHandler, params, actionPath);
      };

      next(initialState, wrappedResetFn, wrappedActionFn);
    };
  };
};