export function warning(condition, message) {
  if (!condition && typeof console !== undefined) {
    console.error('Warning: ' + message)
    try {
      // This error was thrown as a convenience so that you can use this stack
      // to find the callsite that caused this warning to fire.
      throw new Error(message)
    } catch (e) {}
  }
}

export function mapObjIndexed(fn, obj) {
  return Object.keys(obj || {}).reduce(
    (r, key) => Object.assign(r, { [key]: fn(obj[key], key) }),
    {},
  )
}

export function mapObj(fn, obj) {
  return Object.keys(obj || {}).reduce(
    (r, key) => Object.assign(r, { [key]: fn(obj[key]) }),
    {},
  )
}

export function compose(...fns) {
  const outerFn = fns.pop()

  return (...args) => fns.reduceRight((acc, fn) => fn(acc), outerFn(...args))
}

// XXX: wrap in currying goodness
// XXX: check transpilation of { ...obj }
export const assoc = p => (val, obj) => ({ ...obj, [p]: val })
export const prop = p => obj => obj[p]

export const isIterator = o => o && typeof o.next === 'function'

// Simple van Laarhoven lens implementation

const fmap = (fn, functor) => functor.map(fn)

const Identity = val => ({
  val,
  map: f => Identity(f(val)),
})

const Constantly = val => ({
  val,
  map: function(f) {
    return this
  },
})

export const lens = (getter, setter) => vf => in_ =>
  fmap(focus => setter(focus, in_), vf(getter(in_)))

export const view = (lens, x) => lens(Constantly)(x).val
export const over = (lens, f, x) => lens(v => Identity(f(v)))(x).val
export const set = (lens, v, x) => over(lens, () => v, x)

export const lensProp = p => lens(prop(p), assoc(p))
