import test from 'tape'
import { lens, view, over, set, lensProp, compose } from '../src/internal/util'

const fixt1 = {
  a: {
    b: {
      c: 55,
      d: 66,
      e: {
        foo: 'bar',
      },
    },
  },
  aa: 'aa',
  bb: {
    greeting: 'hi',
  },
}

test('simple lensProp view', function(t) {
  const l = lensProp('aa')

  t.equal(view(l, fixt1), "aa")

  t.end()
})

test('simple lensProp set', function(t) {
  const l = lensProp('aa')

  t.equal(view(l, fixt1), "aa")

  const res1 = set(l, "abc", fixt1)
  t.equal(view(l, res1), "abc")
  t.deepEqual(res1, { ...fixt1, aa: "abc" })

  t.end()
})

test('set uses structural sharing', function(t) {
  const l = lensProp('aa')

  const res1 = set(l, "abc", fixt1)

  t.equal(res1.a, fixt1.a)

  t.end()
})

test('simple lensProp over', function(t) {
  const l = lensProp('aa')

  const res1 = over(l, v => `${v}bbcc`, fixt1)
  t.equal(view(l, res1), "aabbcc")
  t.deepEqual(res1, { ...fixt1, aa: "aabbcc" })

  t.end()
})

test('set does not change the underlying object', function(t) {
  const x = Object.assign({}, fixt1)
  const l = lensProp('aa')

  const res1 = set(l, "abc", x)

  t.deepEqual(x, fixt1)

  t.end()
})

test('view on composed lenses', function(t) {
  const l = compose(lensProp('a'), lensProp('b'), lensProp('c'))

  t.equal(view(l, fixt1), 55)

  t.end()
})

test('set on composed lenses', function(t) {
  const l = compose(lensProp('a'), lensProp('b'), lensProp('c'))

  const res1 = set(l, 99, fixt1)
  t.equal(res1.a.b.c, 99)
  t.equal(view(l, res1), 99)

  t.end()
})

test('over on composed lenses', function(t) {
  const l = compose(lensProp('a'), lensProp('b'), lensProp('d'))

  const res1 = over(l, v => v+11, fixt1)
  t.equal(res1.a.b.d, 77)
  t.equal(view(l, res1), 77)

  t.end()
})

test('structural sharing on composed lenses', function(t) {
  const l = compose(lensProp('a'), lensProp('b'), lensProp('c'))

  const res1 = set(l, 99, fixt1)
  t.equal(res1.aa, fixt1.aa)
  t.equal(res1.bb, fixt1.bb)
  t.notEqual(res1.a, fixt1.a)
  t.notEqual(res1.a.b, fixt1.a.b)
  t.notEqual(res1.a.b.c, fixt1.a.b.c)
  t.equal(res1.a.b.d, fixt1.a.b.d)
  t.equal(res1.a.b.e, fixt1.a.b.e)

  t.end()
})
