import test from 'tape'
import sinon from 'sinon'
import { atom, swap, reset, deref, dispatch } from '../src/index'
import { applyLogger } from '../src/index'

const mkAtomFixture1 = middleware => atom(middleware, {
  initialState: {
    a: 1,
  },
  modules: {
    b: {
      initialState: {
        val: 3,
      },
      actions: {
        decBy: function(scope, decQty) {
          swap(scope, oldState => ({ ...oldState, val: oldState.val - decQty }))
        }
      },
    },
  },
  actions: {
    incBy: function(scope, incQty) {
      swap(scope, oldState => ({ ...oldState, a: oldState.a + incQty }))
    }
  },
})

test('stateLogger initial call', function(t) {
  const stub = sinon.stub()

  const a = mkAtomFixture1(applyLogger(stub, null))

  t.equal(stub.callCount, 1)
  t.ok(stub.firstCall.calledWith({}, a.deref()))

  t.end()
})

test('stateLogger call after change', function(t) {
  const stub = sinon.stub()

  const a = mkAtomFixture1(applyLogger(stub, null))

  const oldState = a.deref()
  dispatch(a, 'b.decBy', 1)
  const newState = a.deref()

  t.equal(stub.callCount, 2)
  t.ok(stub.secondCall.calledWith(oldState, newState))

  t.end()
})

test('actionLogger call after dispatch', function(t) {
  const stub = sinon.stub()

  const a = mkAtomFixture1(applyLogger(null, stub))

  dispatch(a, 'b.decBy', 3)

  t.equal(stub.callCount, 1)
  t.ok(stub.firstCall.calledWith(a, 'b.decBy', [3]))

  t.end()
})
