import { JSDOM } from 'jsdom'

const { document } = (new JSDOM(`...`)).window;
global.window = document.defaultView;
Object.keys(document.defaultView).forEach((property) => {
  if (typeof global[property] === 'undefined') {
    global[property] = document.defaultView[property];
  }
});

global.navigator = {
  userAgent: 'node.js'
};

import test from 'tape'
import { shallow, mount } from 'enzyme'
import { atom, swap, reset, deref, dispatch } from '../src/index'
import { Provider, connect } from '../src/internal/react'
import { mkAtomFixture1 } from './fixtures'

import React from 'react'

function mkFixture1() {
  const a = mkAtomFixture1()

  const mapState = (state, ownProps) => ({
    label: "Count: " + state.b.val,
  })

  const mapActions = (atom, ownProps) => ({
    onClick: () => atom.dispatch('b.decBy', 1),
  })

  const ConnectedButton = connect(mapState, mapActions)(CoolButton)

  const tree = mount(
    <Provider stateAtom={a}>
      <ConnectedButton></ConnectedButton>
    </Provider>
  )

  return [a, tree]
}

const CoolButton = props => (<button onClick={props.onClick}>{props.label}</button>)

test('basic mounting of provider & connected component', function(t) {
  const [a, tree] = mkFixture1()

  t.equal(tree.find('button').html(), "<button>Count: 3</button>")
  t.end()
})

test('action causes dispatch', function(t) {
  const [a, tree] = mkFixture1()

  tree.find('button').simulate('click')

  t.equal(tree.find('button').html(), "<button>Count: 2</button>")
  t.equal(a.deref().b.val, 2)
  t.end()
})

test('swap causes rerender', function(t) {
  const [a, tree] = mkFixture1()

  swap(a, oldState => ({ ...oldState, b: { val: 9 }}))

  t.equal(tree.find('button').html(), "<button>Count: 9</button>")
  t.equal(a.deref().b.val, 9)

  t.end()
})

test('dispatch causes rerender', function(t) {
  const [a, tree] = mkFixture1()

  dispatch(a, 'b.decBy', 2)

  t.equal(tree.find('button').html(), "<button>Count: 1</button>")
  t.equal(a.deref().b.val, 1)

  t.end()
})
